from django.core.management.base import BaseCommand
from google_auth_oauthlib.flow import InstalledAppFlow
from django.conf import settings
import json
import os
import tempfile


# OAuth 2.0 스코프 정의
SCOPES = ['https://www.googleapis.com/auth/gmail.readonly', 'https://www.googleapis.com/auth/gmail.send']

class Command(BaseCommand):
    help = 'Launches a browser to authenticate the user and obtain a refresh token for Gmail API'

    def handle(self, *args, **kwargs):
        client_id = settings.GMAIL_OAUTH_CLIENT_ID
        client_secret = settings.GMAIL_OAUTH_CLIENT_SECRET

        # 임시 파일을 생성하여 OAuth 2.0 클라이언트 시크릿 정보를 담음
        with tempfile.NamedTemporaryFile(delete=False, mode='w+', suffix='.json') as temp_client_secret_file:
            client_config = {
                "installed": {
                    "client_id": client_id,
                    "client_secret": client_secret,
                    "auth_uri": "https://accounts.google.com/o/oauth2/auth",
                    "token_uri": "https://oauth2.googleapis.com/token",
                    "redirect_uris": ["http://localhost"]
                }
            }

            # 임시 파일에 클라이언트 정보 작성
            json.dump(client_config, temp_client_secret_file)
            temp_client_secret_file_path = temp_client_secret_file.name

        try:
            # OAuth 인증 플로우 시작
            flow = InstalledAppFlow.from_client_secrets_file(temp_client_secret_file_path, SCOPES)
            creds = flow.run_local_server(port=8912)

            # refresh token 저장
            try:
                from gmail_oauth_backend.models import RefreshToken
                RefreshToken.objects.update_or_create(
                    key='PyLab',
                    defaults={'value': json.loads(creds.to_json())}
                )
            except Exception as e:
                self.stderr.write(self.style.ERROR(f'Can not save Refresh Token to database: {e}'))

            self.stdout.write(self.style.SUCCESS(f'Refresh Token: {creds.refresh_token}'))

        finally:
            # 임시 파일 삭제
            if os.path.exists(temp_client_secret_file_path):
                os.remove(temp_client_secret_file_path)



import base64
import logging

from django.conf import settings
from django.core.mail.backends.base import BaseEmailBackend

import google.oauth2.credentials
import googleapiclient.discovery


logger = logging.getLogger(__name__)


class GmailOauthBackend(BaseEmailBackend):
    def __init__(self, client_id=None, client_secret=None, user_id=None, fail_silently=False, **kwargs):
        super().__init__(fail_silently=fail_silently, **kwargs)
        self.client_id = client_id or settings.GMAIL_OAUTH_CLIENT_ID
        self.client_secret = client_secret or settings.GMAIL_OAUTH_CLIENT_SECRET

        if hasattr(settings, 'GMAIL_OAUTH_USER_ID'):
            self.user_id = user_id or settings.GMAIL_API_USER_ID
        else:
            self.user_id = user_id or 'me'

        try:
            from gmail_oauth_backend.models import RefreshToken
            self.refresh_token = RefreshToken.objects.get(key='PyLab').value['refresh_token']
        except Exception:
            raise ValueError('Refresh Token does not exist in the database. Please run the "init_gmail_oauth_token" management command to obtain a refresh token.')

        credentials = google.oauth2.credentials.Credentials(
            'token',
            refresh_token=self.refresh_token,
            token_uri='https://accounts.google.com/o/oauth2/token',
            client_id=self.client_id,
            client_secret=self.client_secret
        )
        if not credentials or not credentials.valid:
            raise ValueError('Credentials are invalid or expired. Please run the "init_gmail_oauth_token" management command to obtain a refresh token.')

        self.service = googleapiclient.discovery.build('gmail', 'v1', credentials=credentials, cache_discovery=False)

    def send_message(self, email_message):
        if not email_message.recipients():
            return False
        message = email_message.message()
        if email_message.bcc:
            email_message._set_list_header_if_not_empty(message, 'Bcc', email_message.bcc)
        raw_message = {'raw': base64.urlsafe_b64encode(message.as_bytes()).decode()}
        return self.service.users().messages().send(userId=self.user_id, body=raw_message)

    def send_messages(self, email_messages):
        if not email_messages:
            return 0
        msg_count = 0
        last_exception = None

        def send_callback(r_id, response, exception):
            nonlocal msg_count, last_exception
            if exception is not None:
                logger.exception('An error occurred sending the message via GMail API:  %s', exception)
                last_exception = exception
            else:
                msg_count += 1

        batch = self.service.new_batch_http_request(send_callback)
        for message in email_messages:
            batch.add(self.send_message(message))
        batch.execute()
        if not self.fail_silently and last_exception:
            raise last_exception
        return msg_count



from django.apps import AppConfig


class GmailOauthBackendConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'gmail_oauth_backend'
    verbose_name = 'Gmail OAuth Backend'


# Generated by Django 5.1.1 on 2024-09-11 03:51

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='RefreshToken',
            fields=[
                ('key', models.CharField(max_length=5, primary_key=True, serialize=False, unique=True)),
                ('value', models.JSONField()),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
            ],
            options={
                'verbose_name': 'Gmail OAuth Refresh Token',
                'verbose_name_plural': 'Gmail OAuth Refresh Tokens',
            },
        ),
    ]



from django.contrib import admin

from gmail_oauth_backend.models import RefreshToken


@admin.register(RefreshToken)
class RefreshTokenAdmin(admin.ModelAdmin):
    list_display = ['get_refresh_token', 'created', 'updated']

    def get_fields(self, request, obj=None):
        fields = super().get_fields(request, obj)
        if obj:
            fields = [field for field in fields if field != 'key']
        return fields

    def get_refresh_token(self, obj):
        return obj.value.get('refresh_token', '-')  # 'age' 키가 없으면 'N/A' 출력
    get_refresh_token.short_description = 'Refresh Token'
